#include <algorithm>

#include "AI.h"

#include "WorldManager.h"
#include "LogManager.h"
#include "Event.h"
#include "StepEvent.h"
#include "Node.h"
#include "NodeMap.h"

AI::AI(int player, Difficulty diff, NodeMap *newMap) {
	playerNumber = player;
	difficulty = diff;
	behavior = MINES;
	decision_countdown = 0;
	map = newMap;
	
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, STEP_EVENT);
	
	setType("AI");
	setSolid(false);
}

AI::~AI() {
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, STEP_EVENT);
}

int AI::eventHandler(Event *e) {
	if (e->getType() == STEP_EVENT) {
		step();
		return 1;
    }
	return 0;
}

void AI::draw() {
	
}

void AI::step() {
	decision_countdown--;
    if (decision_countdown <= 0) {
		setBehavior();
		decision_countdown = difficulty;
		takeMove();
	}else if(difficulty == HARD)
		takeMove();
}

void AI::setBehavior() {
	if(map->getResources(playerNumber) < COST_S_BUILDING)
		behavior = MINES;
	else {
		Node ***currMap = map->getMap();
		int swords=0;
		int pike=0;
		int horse=0;
		for(int i = 0; i < map->getMapX(); i++)
			for(int j=0; j < map->getMapY(); j++)
					if(currMap[i][j]->getOwner() != playerNumber) {
						swords += currMap[i][j]->getSword();
						pike += currMap[i][j]->getPike();
						horse += currMap[i][j]->getHorse();
					}
		int target = max(swords,max(horse,pike));					

		if(target == swords)
			behavior = HORSE;
		else if(target == pike)
			behavior = SWORD;
		else
			behavior = PIKE;
	}	
}

void AI::takeMove() {
	switch(behavior) {
		case MINES:
			makeMine();
			break;
		case SWORD:
			makeSword();
			break;
		case PIKE:
			makePike();
			break;
		case HORSE:
			makeHorse();
			break;
		default:
			makePike();
	}
	moveUnits();
	
}

void AI::moveUnits() {
	Node **ownedNodes;
	int num_nodes = map->getOwnedNodes(playerNumber, ownedNodes);

	for(int i = 0; i < num_nodes; i++)
	{
		int swords = ownedNodes[i]->getSword();
		int pikes = ownedNodes[i]->getPike();
		int horses = ownedNodes[i]->getHorse();
		int target = max(swords, max(horses, pikes));
		Node::Unit target_unit;
		if(target == swords)
			target_unit = Node::SWORDSMEN;
		else if(target == pikes)
			target_unit = Node::PIKEMEN;
		else
			target_unit = Node::HORSEMEN;
		
		Node *unowned_nodes[4];
		for(int j = 0; j < 4; j++)
			unowned_nodes[j] = NULL;
		
		Road *south_road = ownedNodes[i]->getRoad(Node::SOUTH);
		Node *south = NULL;
		if(south_road != NULL)
			south = south_road->getNode(Node::SOUTH);
		Road *north_road = ownedNodes[i]->getRoad(Node::NORTH);
		Node *north = NULL;
		if(north_road != NULL)
			north = north_road->getNode(Node::NORTH);
		Road *east_road = ownedNodes[i]->getRoad(Node::EAST);
		Node *east = NULL;
		if(east_road != NULL)
			east = east_road->getNode(Node::EAST);
		Road *west_road = ownedNodes[i]->getRoad(Node::WEST);
		Node *west = NULL;
		if(west_road != NULL)
			west = west_road->getNode(Node::WEST);
		
		if(south != NULL && south->getOwner() != playerNumber)
			unowned_nodes[Node::SOUTH] = south;
		if(north != NULL && north->getOwner() != playerNumber)
			unowned_nodes[Node::NORTH] = north;
		if(east != NULL && east->getOwner() != playerNumber)
			unowned_nodes[Node::EAST] = east;
		if(west != NULL && west->getOwner() != playerNumber)
			unowned_nodes[Node::WEST] = west;
		
		for(int j = 0; j < 4; j++)
		{
			if(unowned_nodes[j] != NULL)
			{
				if(rand() % 3 == 0)
					ownedNodes[i]->moveUnits(target_unit,j);
			}else if(j == 3)
			{
				if(rand() % 2 == 0)
					ownedNodes[i]->moveUnits(target_unit,Node::NORTH);
				else
					ownedNodes[i]->moveUnits(target_unit,Node::WEST);	
			}
		}
	}	
}//moveUnits

void AI::makeMine() {
	LogManager &lm = LogManager::getInstance();

	Node **owned;
	int ownedSize = map->getOwnedNodes(playerNumber, owned);

	for(int i =0; i < ownedSize; i++) {
		if(owned[i]->getBuilding(0) == NO_BUILDING || owned[i]->getBuilding(1) == NO_BUILDING)
		{
			if(map->takeResources(playerNumber, owned[i]->checkBuildingCost(MINE))) 
				owned[i]->addBuilding(MINE);
		}
	}
}

void AI::makeSword() {
	LogManager &lm = LogManager::getInstance();

	Node **owned;
	int ownedSize = map->getOwnedNodes(playerNumber,owned);

	for(int i =0; i < ownedSize; i++) {
		if(owned[i]->getBuilding(0) == NO_BUILDING || owned[i]->getBuilding(1) == NO_BUILDING)
		{
			if(map->takeResources(playerNumber, owned[i]->checkBuildingCost(SWORDSMEN_BARRACKS))) 
				owned[i]->addBuilding(SWORDSMEN_BARRACKS);
		}
	}	
}

void AI::makePike() {
	LogManager &lm = LogManager::getInstance();

	Node **owned;
	int ownedSize = map->getOwnedNodes(playerNumber,owned);

	for(int i =0; i < ownedSize; i++) {
		if(owned[i]->getBuilding(0) == NO_BUILDING || owned[i]->getBuilding(1) == NO_BUILDING)
		{
			if(map->takeResources(playerNumber, owned[i]->checkBuildingCost(PIKEMEN_BARRACKS))) 
				owned[i]->addBuilding(PIKEMEN_BARRACKS);
		}
	}
}

void AI::makeHorse() {
	LogManager &lm = LogManager::getInstance();

	Node **owned;
	int ownedSize = map->getOwnedNodes(playerNumber,owned);

	for(int i =0; i < ownedSize; i++) {
		if(owned[i]->getBuilding(0) == NO_BUILDING || owned[i]->getBuilding(1) == NO_BUILDING)
		{
			if(map->takeResources(playerNumber, owned[i]->checkBuildingCost(HORSEMEN_BARRACKS))) 
				owned[i]->addBuilding(HORSEMEN_BARRACKS);
		}
	}
}
