#include "Menu.h"
#include "Node.h"
#include "AI.h"
#include "NodeMap.h"
#include "Event.h"
#include "StepEvent.h"
#include "KeyboardEvent.h"
#include "WorldManager.h"
#include "InputManager.h"
#include "GraphicsManager.h"

Menu::Menu(string message) : TextMessage(message){
	 setType("Menu");
	 
	 difficulty = AI::EASY;
	 boardSize = 4;
	 
	 InputManager &im = InputManager::getInstance();
	 im.registerInterest(this, KEYBOARD_EVENT);
}

Menu::~Menu() {
	int map_x = boardSize;
	int map_y = boardSize;

	Node ***map = (Node ***)malloc(sizeof(Node **) *map_x);
	
	for(int i = 0; i < map_x; i++)
		map[i] = (Node **)malloc(sizeof(Node *) * map_y);
		
	for(int x = 0; x < map_x; x++)
		for(int y = 0; y < map_y; y++)
			map[x][y] = new Node(x,y);
		
	//add vertical roads
	for(int x = 0; x < map_x; x++)
		for(int y = 0; y < map_y-1; y++)
			new Road(true, map[x][y], map[x][y+1]);
			
	//add horizontal roads
	for(int x = 0; x < map_x-1; x++)
		for(int y = 0; y < map_y; y++)
			new Road(false, map[x+1][y], map[x][y]);
			
	map[0][0]->addUnit(Node::SWORDSMEN, PLAYER_OWNED);
	map[map_x-1][map_y-1]->addUnit(Node::SWORDSMEN, AI_OWNED);
	map[0][0]->setIsSelected(true);

	NodeMap * nodeMap = new NodeMap(map[0][0], map, map_x*map_y, map_x, map_y);
	new AI(PLAYER+1, difficulty,nodeMap);

	// prevent the destructors from being called
	//map = NULL;
	
	InputManager &im = InputManager::getInstance();
	im.unregisterInterest(this, KEYBOARD_EVENT);
}

int Menu::eventHandler(Event *e) {
    if (e->getType() == KEYBOARD_EVENT) {
		KeyboardEvent *p_keyboard_event = static_cast <KeyboardEvent *> (e);
		kbd(p_keyboard_event);
		return 1;
    }
	
	if (e->getType() == STEP_EVENT) {
		return 1;
    }
	return 0;
}

void Menu::kbd(KeyboardEvent *p_keyboard_event) {
	switch(p_keyboard_event->getKey()) {
	case 'e':
		difficulty = AI::EASY;
		break;
	case 'm':
		difficulty = AI::MEDIUM;
		break;
	case 'h':
		difficulty = AI::HARD;
		break;
	case '2':
		boardSize = 2;
		break;
	case '3':
		boardSize = 3;
		break;	
	case '4':
		boardSize = 4;
		break;	
	case '5':
		boardSize = 5;
		break;	
	case '6':
		boardSize = 6;
		break;	
	case '7':
		boardSize = 7;
		break;
	default:
		WorldManager &gw = WorldManager::getInstance();
		gw.markForDelete(this);
	}
}

void Menu::draw() {
	GameObject::draw();
	
	GraphicsManager &gm = GraphicsManager::getInstance();
	
	char buffer[15];
	
	sprintf(buffer, "%d by %d grid -- ",boardSize,boardSize);
	
	string output= buffer;
	
	switch(difficulty) {
		case AI::EASY:
			output += "Easy AI";
			break;
		case AI::MEDIUM:
			output += "Medium AI";
			break;
		case AI::HARD:
			output += "Hard AI";
			break;
	};
	
	int y=0;
	for(int x = gm.getHorizontal()/2-10; x < output.length()+ gm.getHorizontal()/2-10; x++)
	{
		gm.drawCh(Position(x,gm.getVertical()/2+19), output[y++]);
	}
}
