#include <cstdlib>

#include "NodeMap.h"

#include "InputManager.h"
#include "KeyboardEvent.h"
#include "WorldManager.h"
#include "GameManager.h"
#include "GraphicsManager.h"
#include "LogManager.h"
#include "StepEvent.h"
#include "GameOver.h"

#include "sstream"

NodeMap::NodeMap(Node *new_selected, Node ***level, int count, int x, int y) {
	selectedNode = new_selected;
	map = level;
	nodeCount=count;
	gridX= x;
	gridY= y;
	for(int i = 0; i < gridX; i++)
		for(int j = 0; j < gridY; j++)
		{
			if(map[i][j] == selectedNode)
			{
				selectedX = i;
				selectedY = j;
			}
		}
	selectedUnit = Node::NONE;
	
	step_slowdown = STEP_SLOWDOWN;
	step_countdown = step_slowdown;
	
	InputManager &im = InputManager::getInstance();
	im.registerInterest(this, KEYBOARD_EVENT);
	WorldManager &wm = WorldManager::getInstance();
	wm.registerInterest(this, STEP_EVENT);
	
	setType("NodeMap");
	setSolid(false);
		
	//set resources
	for(int i = 0; i < NUM_PLAYERS+1; i++)
	{
		resources[i] = 100;
	}
}

NodeMap::~NodeMap() {
	InputManager &im = InputManager::getInstance();
	im.unregisterInterest(this, KEYBOARD_EVENT);
	WorldManager &wm = WorldManager::getInstance();
	wm.unregisterInterest(this, STEP_EVENT);
}

int NodeMap::eventHandler(Event *e) {
    if (e->getType() == KEYBOARD_EVENT) {
		KeyboardEvent *p_keyboard_event = static_cast <KeyboardEvent *> (e);
		kbd(p_keyboard_event);
		return 1;
    }
    if (e->getType() == STEP_EVENT) {
		if(checkGameOver()) {
			new GameOver("gameover");
			return 1;
		}
		StepEvent *p_step_event = static_cast <StepEvent *> (e);
		for(int i =0; i < gridX; i++)
			for(int j =0; j < gridY; j++)
				map[i][j]->eventHandler(p_step_event);
	
		step_countdown--;
		
		if(step_countdown % 10 == 0)
		{
			for(int i=0; i < NUM_PLAYERS+1; i++)
				resources[i] += PASSIVE_RESOURCE_INCOME;
		}
		
		if(step_countdown > 0)
			return 1;
		step_countdown = step_slowdown;	
		
		step();
		return 1;
    }
	return 0;
}

bool NodeMap::checkGameOver() {
	int winner = map[0][0]->getOwner();
	for(int x = 0; x < gridX; x++)
		for(int y = 0; y < gridY; y++)
		{
			if(map[x][y]->getOwner() != NOT_OWNED)
			{
				if(winner != map[x][y]->getOwner())
				{	
					return false;
				}
			}
		}
	return true;
}

void NodeMap::step() {
	for(int i =0; i < gridX; i++)
		for(int j =0; j < gridY; j++)
		{
			int map_owner = map[i][j]->getOwner();
			if(map_owner > 0 && map_owner < NUM_PLAYERS+1)
			{
				giveResources(map_owner, map[i][j]->getResources());
				if(getResources(map_owner) > 0)
					map[i][j]->buildingStep();
				else
					setResources(map_owner, 0);
			}
		}
}

void NodeMap::kbd(KeyboardEvent *p_keyboard_event) {
	#ifdef DEBUG3
		LogManager &lm = LogManager::getInstance();
	#endif
	GameManager &gm = GameManager::getInstance();
	
	switch(p_keyboard_event->getKey()) {
	case KEY_UP:
		if(selectedY!=0) {
			selectedNode->setIsSelected(false);
			selectedY--;
			#ifdef DEBUG3
				lm.writeLog("NodeMap::kbd: Moving to map location %d,%d", selectedX, selectedY);
			#endif
			selectedNode = map[selectedX][selectedY];
			selectedNode->setIsSelected(true);
		}
		break;
	case KEY_DOWN:
		if(selectedY < gridY - 1) {
			selectedNode->setIsSelected(false);
			selectedY++;
			#ifdef DEBUG3
				lm.writeLog("NodeMap::kbd: Moving to map location %d,%d", selectedX, selectedY);
			#endif
			selectedNode = map[selectedX][selectedY];
			selectedNode->setIsSelected(true);
		}
		break;
	case KEY_LEFT:
		if(selectedX!=0) {
			selectedNode->setIsSelected(false);
			selectedX--;
			#ifdef DEBUG3
				lm.writeLog("NodeMap::kbd: Moving to map location %d,%d", selectedX, selectedY);
			#endif
			selectedNode = map[selectedX][selectedY];
			selectedNode->setIsSelected(true);
		}
		break;
	case KEY_RIGHT:
		if(selectedX < gridX -1) {
			selectedNode->setIsSelected(false);
			selectedX++;
			#ifdef DEBUG3
				lm.writeLog("NodeMap::kbd: Moving to map location %d,%d", selectedX, selectedY);
			#endif
			selectedNode = map[selectedX][selectedY];
			selectedNode->setIsSelected(true);
		}
		break;
	case 'w':
		if(selectedNode->getOwner() == PLAYER_OWNED)
			selectedNode->moveUnits(selectedUnit,Node::NORTH);
		break;
	case 'a':
		if(selectedNode->getOwner() == PLAYER_OWNED)
			selectedNode->moveUnits(selectedUnit,Node::WEST);
		break;
	case 's':
		if(selectedNode->getOwner() == PLAYER_OWNED)
			selectedNode->moveUnits(selectedUnit,Node::SOUTH);
		break;
	case 'd':
		if(selectedNode->getOwner() == PLAYER_OWNED)
			selectedNode->moveUnits(selectedUnit,Node::EAST);
		break;
	case 'z':
		if(selectedUnit == Node::SWORDSMEN) {
			if(selectedNode->getOwner() == PLAYER_OWNED && takeResources(PLAYER, selectedNode->checkBuildingCost(SWORDSMEN_BARRACKS))) 
				selectedNode->addBuilding(SWORDSMEN_BARRACKS);
				//selectedNode->addUnit(Node::SWORDSMEN,1);
		}
		else 
			selectedUnit = Node::SWORDSMEN;
		break;
	case 'x':
		if(selectedUnit == Node::HORSEMEN) {
			if(selectedNode->getOwner() == PLAYER_OWNED && takeResources(PLAYER, selectedNode->checkBuildingCost(HORSEMEN_BARRACKS))) 
				selectedNode->addBuilding(HORSEMEN_BARRACKS);
				//selectedNode->addUnit(Node::HORSEMEN,1);
		}
		else
			selectedUnit = Node::HORSEMEN;
		break;
	case 'c':
		if(selectedUnit == Node::PIKEMEN) {
			if(selectedNode->getOwner() == PLAYER_OWNED && takeResources(PLAYER, selectedNode->checkBuildingCost(PIKEMEN_BARRACKS))) 
				selectedNode->addBuilding(PIKEMEN_BARRACKS);
				//selectedNode->addUnit(Node::PIKEMEN,1);
		}
		else
			selectedUnit = Node::PIKEMEN;
		break;
	case 'v':
		if(selectedUnit == Node::NONE) {
			if(selectedNode->getOwner() == PLAYER_OWNED)
				giveResources(PLAYER, selectedNode->removeBuildings());
				//selectedNode->addUnit(Node::PIKEMEN,1);
		}
		else
			selectedUnit = Node::NONE;
		break;
	case 'm':
		if(selectedNode->getOwner() == PLAYER_OWNED && takeResources(PLAYER, selectedNode->checkBuildingCost(MINE)))
			selectedNode->addBuilding(MINE);
		break;
	case 'p':
		gm.setGameOver(true);
		break;
	default:
		;
		#ifdef DEBUG3
			lm.writeLog("NodeMap::kbd: Unrecognized keyboard key pressed: key #%d", p_keyboard_event->getKey());
		#endif
	}
	return;
}//kbd

int NodeMap::getMapX() {
	return gridX;
}

int NodeMap::getMapY() {
	return gridY;
}

Node ***NodeMap::getMap() {
	return map;
}

int NodeMap::getOwnedNodes(int player, Node **&nodeMap) {
	Node **owned = (Node **)malloc(sizeof(NodeMap *)*getMapX()*getMapY());
	int ownedSize=0;
	for(int i =0; i < getMapX(); i++) 
		for(int j =0;j< getMapY(); j++)
			if(getMap()[i][j]->getOwner() == player) {
				owned[ownedSize++] = getMap()[i][j];
			}
	nodeMap = owned;
	return ownedSize;
}

// arguments: player - index for the player, if accessing the player's resources pass PLAYER,
//					 -	otherwise pass a number from 0-NUM_PLAYERS exclusive
int NodeMap::getResources(int player) {
	if(player > 0 && player < NUM_PLAYERS+1)
		return resources[player];
	else
		return 0;
}//getResources

void NodeMap::setResources(int player, int new_resource) {
	if(player > 0 && player < NUM_PLAYERS+1)
		resources[player] = new_resource;
		
	if(resources[player] > MAX_RESOURCES)
		resources[player] = MAX_RESOURCES;
}//setResources

void NodeMap::giveResources(int player, int num_resources) {
	if(player > 0 && player < NUM_PLAYERS+1)
		resources[player] += num_resources;
		
	if(resources[player] > MAX_RESOURCES)
		resources[player] = MAX_RESOURCES;
}//giveResources

// return: true if resources were taken, and false if there weren't enough resources
bool NodeMap::takeResources(int player, int num_resources) {
	if(player > 0 && player < NUM_PLAYERS+1)
	{
		if(resources[player] < num_resources)
			return false;
		
		resources[player] -= num_resources;
		return true;
	}else
		return false;
}//takeResources

void NodeMap::draw() {
	GraphicsManager &gm = GraphicsManager::getInstance();
	
	// draw the available resources
	std::ostringstream sout;
	sout << "Player:\t";
	if(resources[PLAYER] < 1000)
		sout << "0";
	if(resources[PLAYER] < 100)
		sout << "0";
	if(resources[PLAYER] < 10)
		sout << "0";
	sout << resources[PLAYER] << "   ";
	
	for(int player_index = PLAYER+1; player_index < NUM_PLAYERS+1; player_index++)
	{
		sout << "AI:\t";
		if(resources[player_index] < 1000)
			sout << "0";
		if(resources[player_index] < 100)
			sout << "0";
		if(resources[player_index] < 10)
			sout << "0";
		sout << resources[player_index] << "   ";
	}
	
	string UI_str = sout.str() + "\n";
	
	for(int x = 0; x < UI_str.length(); x++)
	{
		gm.drawCh(Position(x,0), UI_str[x]);
	}
	
	// draw controls
	string UI_line_0 = "Controls:";
	string UI_line_1 = "  z - Swordsmen (150/-50)   w - send Units North";
	string UI_line_2 = "  x - Horsemen  (160/-40)   s - send Units South";
	string UI_line_3 = "  c - Pikemen   (190/-30)   a - send Units West";
	string UI_line_4 = "  m - Mine      (090/+40)   d - send Units East";
	string UI_line_5 = "  v - Clear";
	
	int UI_line_1_color = AI_OWNED_COLOR;
	int UI_line_2_color = AI_OWNED_COLOR;
	int UI_line_3_color = AI_OWNED_COLOR;
	int UI_line_5_color = AI_OWNED_COLOR;
	
	switch(selectedUnit)
	{
		case Node::SWORDSMEN:
			UI_line_1_color = PLAYER_OWNED_COLOR;
			break;
		case Node::HORSEMEN:
			UI_line_2_color = PLAYER_OWNED_COLOR;
			break;
		case Node::PIKEMEN:
			UI_line_3_color = PLAYER_OWNED_COLOR;
			break;
		case Node::NONE:
			UI_line_5_color = PLAYER_OWNED_COLOR;
	};
	
	for(int x = 0; x < UI_line_1.length(); x++)
	{
		if(x < UI_line_0.length())
			gm.drawCh(Position(x,1), UI_line_0[x]);
		if(x < UI_line_1.length())
			gm.drawCh(Position(x,2), UI_line_1[x], UI_line_1_color);
		if(x < UI_line_2.length())
			gm.drawCh(Position(x,3), UI_line_2[x], UI_line_2_color);
		if(x < UI_line_3.length())
			gm.drawCh(Position(x,4), UI_line_3[x], UI_line_3_color);
		if(x < UI_line_4.length())
			gm.drawCh(Position(x,5), UI_line_4[x]);
		if(x < UI_line_5.length())
			gm.drawCh(Position(x,6), UI_line_5[x], UI_line_5_color);			
	}
}//draw
