/*
	game.cpp
	Written by Cordell Zebrose
	
	This is the main file for the saucer shoot test game.
*/

#include "LogManager.h"
#include "GameManager.h"
#include "WorldManager.h"
#include "GraphicsManager.h"
#include "ResourceManager.h"
#include "Clock.h"
#include "GameObject.h"
#include "GameObjectList.h"
#include "GameObjectListIterator.h"
#include <string>
#include <cstdlib>

#include "Menu.h"

#define APPEND_LOGFILE false
#define FLUSH_AFTER_LOGWRITE true

using namespace std;

// function prototypes
void populateWorld();
void loadResources();

int main(int argc, char *argv[]) {	
	LogManager &lm = LogManager::getInstance();
	
	// start up the various Managers
	GameManager &gm = GameManager::getInstance();
	if(gm.startup(APPEND_LOGFILE, FLUSH_AFTER_LOGWRITE)) {
		printf("Error: starting Game Manager\n");
		gm.shutdown();
		return -1;
	}// if
	
	// Starting Game
	printf("Game Starting...\n");
#ifdef DEBUG1
	lm.writeLog("\t\tGame Starting");
#endif
	
	//-------------------------------------------
	
#ifdef DEBUG1
	lm.writeLog("\t\tLoading Resources");
#endif
	loadResources();
	
	//-------------------------------------------
	
#ifdef DEBUG1
	lm.writeLog("\t\tPopulating World");
#endif
	populateWorld();
	
	//--------------------------------------------
	
#ifdef DEBUG1
	lm.writeLog("\t\tRunning Game");
#endif
	gm.run();	
#ifdef DEBUG1
	lm.writeLog("\t\tGame Ends");
#endif
	printf("...Game Ending\n");
	
	//-----------------------------------------------
	
#ifdef DEBUG1
	lm.writeLog("Shutdown Everything");
#endif
	gm.shutdown();
	return 0;
}// main

//////////////////////////////////////////////////////////////////////////////////////

void loadResources() {
	ResourceManager &resource_manager = ResourceManager::getInstance();
	int err = resource_manager.loadSprite("sprites/node-spr.txt", "node");
	
#ifdef DEBUG1
	if(err != 0)
	{
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("loadResources: Error - there was an error loading the node sprite");
	}//if
#endif

	err = resource_manager.loadSprite("sprites/road-horizontal-spr.txt", "road-horizontal");	
#ifdef DEBUG1
	if(err != 0)
	{
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("loadResources: Error - there was an error loading the road-horizontal sprite");
	}//if
#endif

	err = resource_manager.loadSprite("sprites/road-vertical-spr.txt", "road-vertical");	
#ifdef DEBUG1
	if(err != 0)
	{
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("loadResources: Error - there was an error loading the road-vertical sprite");
	}//if
#endif

	err = resource_manager.loadSprite("sprites/game-start-spr.txt", "menu");
	
#ifdef DEBUG1
	if(err != 0)
	{
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("loadResources: Error - there was an error loading the menu sprite");
	}//if
#endif

	err = resource_manager.loadSprite("sprites/game-end-spr.txt", "gameover");
	
#ifdef DEBUG1
	if(err != 0)
	{
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("loadResources: Error - there was an error loading the gameover sprite");
	}//if
#endif

#ifdef DEBUG3
	LogManager &lm = LogManager::getInstance();
	lm.writeLog("loadResources: There are %d sprites loaded into the game", resource_manager.getSpriteCount());
#endif
} // loadResources

////////////////////////////////////////////////////////////////////////////////////////

void populateWorld() {
	new Menu("menu");
}// populateWorld
