/*
  Clock.h
  Written by Cordell Zebrose
  
  Clock handles timing the pause between frames, by allowing the engine keep track
	of how long it took the last frame to process.
*/

#ifndef _CLOCK_H_
#define _CLOCK_H_

# ifdef __CYGWIN__
#include <windows.h>
#endif
#include <time.h>

class Clock {
	protected:
#ifdef __CYGWIN__
		// if running Cygwin
		LARGE_INTEGER frequency;
		LARGE_INTEGER prev;
#else
		// if running Linux
		struct timespec prev_ts;
#endif
		
	public:
		Clock();
		~Clock();
		
		// returns the number of milliseconds which have past since
		//  	the last time delta() was called, or -1 on error
		long int delta(void);
		
		// busy waits the program for ms milliseconds
		void busySleep(int ms);
};

#endif //_CLOCK_H_
