/*
  GameManager.h
  Written by Cordell Zebrose
  
  GameManager runs the game loop and sets up and shuts down all the other
	Managers.
*/

#ifndef _GAME_MANAGER_H_
#define _GAME_MANAGER_H_

#include "Manager.h"
#include <time.h>

#define FRAME_TIME 33

class GameManager : public Manager {
	private:
		GameManager();						// prevent more GameManagers
		GameManager(const GameManager &gm);		// prevent copying
		void operator=(const GameManager &gm);	// prevent copying
	protected:
		bool game_over;		// true if the game ends
		int time_between_frames; // number of milliseconds allocated for each frame
	public:
		~GameManager();
	
		// returns the singleton instance of the GameManager
		static GameManager &getInstance();
		
		// run the game loop, 
		//	use the arguement for the time between frames, or
		//  FRAME_TIME as a default.
		void run();
		void run(int frame_time);
		
		// starts up all the Managers
		// 		IF append THEN add the log to the end of an
		//			existing log file, otherwise any existing log
		//			file is overwritten.
		//		IF flush THEN after every log message, flush the buffer
		int startup();
		int startup(bool append, bool flush);
		int startup(bool append, bool flush, time_t seed);
		
		// shuts down all the Managers
		void shutdown();
		
		// set the value of game_over, game ends if true
		void setGameOver(bool g_o);
};

#endif //_GAME_MANAGER_H_
