/*
  GameObject.h
  Written by Cordell Zebrose
  
	GameObject is the base class for most user defined objects
*/

#ifndef _GAME_OBJECT_H_
#define _GAME_OBJECT_H_

#include "Position.h"
#include "Event.h"
#include "Sprite.h"
#include "Frame.h"
#include "Box.h"
#include <string>

#define DEFAULT_TYPE "GameObject"

using namespace std;

class GameObject {
	protected:
		Position pos;
		string type;
		int id;
		void init(Position p, string t);
		bool is_solid;
		int altitude;
		Sprite *p_sprite;
		Box bounding_box;
		
		// True if the GameObject's position is in the center of the Sprite
		// False if the GameObject's position is in the upper-left corner of the Sprite
		bool sprite_centered;
		
		// Velocity stuff
		float x_velocity;
		float x_velocity_countdown;
		float y_velocity;
		float y_velocity_countdown;
		
	public:
		// constructors
		GameObject();
		GameObject(Position p);
		GameObject(string type);
		GameObject(Position p, string t);
		virtual ~GameObject();
		
		// modify/get Position field
		void setPos(Position new_pos);
		Position getPos();
		
		// modify/get Type
		void setType(string new_type);
		string getType();
		
		// modify/get id
		void setId(int new_id);
		int getId();
		
		// modify/get is_solid
		void setSolid(bool new_solid);
		bool isSolid();
		
		// modify/get altitude
		void setAltitude(int new_altitude);
		int getAltitude();
		
		// modify/get Sprite
		void setSprite(Sprite *new_sprite);
		Sprite *getSprite();
		
		// modify/get sprite_centered
		void setSpriteCentered(bool new_sprite_centered);
		bool isCentered();
		
		// modify/get bounding_box
		void setBoundingBox(Box new_bounding_box);
		// returns the Bounding Box, accounting for centering
		Box getBoundingBox();
		
		// modift/get velocity x/y
		void setXVelocity(float new_x_velocity);
		void setYVelocity(float new_y_velocity);
		float getXVelocity();
		float getYVelocity();
		
		// perform 1 step of velocity movement
		int getXVelocityStep();
		int getYVelocityStep();
		
		//Handle the events sent to this object
		//  return 0 if the event is ignored, else return 1
		//	default for GameObject is ignore everything
		virtual int eventHandler(Event *p_event);
		
		// Draw this GameObject, the base class does nothing
		//  	with this.
		virtual void draw();
		
};// GameObject

#endif //_GAME_OBJECT_H_
