/*
  GameObjectList.h
  Written by Cordell Zebrose
  
  GameObjectList is a data structure for holding an unordered list of GameObjects
*/

#ifndef _GAME_OBJECT_LIST_H_
#define _GAME_OBJECT_LIST_H_

#define MAX_GAME_OBJECTS 10000

#include "GameObject.h"
#include "GameObjectListIterator.h"

// prototype, so that the two classes can mutually depend on each other
class GameObjectListIterator; 

class GameObjectList {
	protected:
		int count;								// number of objects in list
		GameObject *list[MAX_GAME_OBJECTS];		// an array of pointers to the GameObjects in list
	
	public:
		friend class GameObjectListIterator; 			//< iterators can access 
		GameObjectListIterator createIterator() const; // create an iterator
		
		GameObjectList();
		~GameObjectList();
		
		// Inserts a game object into the list
		// Return 0 if ok, or -1 on error
		int insert(GameObject *p_go);
		
		// Remove a game object from the list
		// Return 0 if removed, 1 if not found, or -1 on error
		int remove(GameObject *p_go);
		
		// Clear the list (setting count to 0)
		void clear();
		
		// Return pointer to object with indicated id,
		//   or NULL if not found
		GameObject *find(int id);
		
		// Return count of number of objects in list
		int getCount();
}; // GameObjectList

#endif //_GAME_OBJECT_LIST_H_

