/*
	GraphicManager.h
	Written by Cordell Zebrose
*/

#ifndef _GRAPHICS_MANAGER_H_
#define	_GRAPHICS_MANAGER_H_

#include "Manager.h"
#include <curses.h>

#define COLOR_DEFAULT 2

// color pairs

#define BLACK_BLACK_PAIR 	1
#define RED_BLACK_PAIR 		2
#define GREEN_BLACK_PAIR 	3
#define YELLOW_BLACK_PAIR 	4
#define BLUE_BLACK_PAIR 	5
#define MAGENTA_BLACK_PAIR 	6
#define CYAN_BLACK_PAIR 	7
#define WHITE_BLACK_PAIR 	8

/*
#define BLACK_RED_PAIR 		9
#define RED_RED_PAIR 		10
#define GREEN_RED_PAIR 		11
#define YELLOW_RED_PAIR 	12
#define BLUE_RED_PAIR 		13
#define MAGENTA_RED_PAIR 	14
#define CYAN_RED_PAIR 		15
#define WHITE_RED_PAIR 		16

#define BLACK_GREEN_PAIR 	17
#define RED_GREEN_PAIR 		18
#define GREEN_GREEN_PAIR 	19
#define YELLOW_GREEN_PAIR 	20
#define BLUE_GREEN_PAIR 	21
#define MAGENTA_GREEN_PAIR 	22
#define CYAN_GREEN_PAIR 	23
#define WHITE_GREEN_PAIR 	24

#define BLACK_YELLOW_PAIR 	25
#define RED_YELLOW_PAIR 	26
#define GREEN_YELLOW_PAIR 	27
#define YELLOW_YELLOW_PAIR 	28
#define BLUE_YELLOW_PAIR 	29
#define MAGENTA_YELLOW_PAIR 30
#define CYAN_YELLOW_PAIR 	31
#define WHITE_YELLOW_PAIR 	32

#define BLACK_BLUE_PAIR 	33
#define RED_BLUE_PAIR 		34
#define GREEN_BLUE_PAIR 	35
#define YELLOW_BLUE_PAIR 	36
#define BLUE_BLUE_PAIR 		37
#define MAGENTA_BLUE_PAIR 	38
#define CYAN_BLUE_PAIR 		39
#define WHITE_BLUE_PAIR 	40

#define BLACK_MAGENTA_PAIR 	41
#define RED_MAGENTA_PAIR 	42
#define GREEN_MAGENTA_PAIR 	43
#define YELLOW_MAGENTA_PAIR	44
#define BLUE_MAGENTA_PAIR 	45
#define MAGENTA_MAGENTA_PAIR46
#define CYAN_MAGENTA_PAIR 	47
#define WHITE_MAGENTA_PAIR 	48

#define BLACK_CYAN_PAIR 	49
#define RED_CYAN_PAIR 		50
#define GREEN_CYAN_PAIR 	51
#define YELLOW_CYAN_PAIR 	52
#define BLUE_CYAN_PAIR 		53
#define MAGENTA_CYAN_PAIR 	54
#define CYAN_CYAN_PAIR 		55
#define WHITE_CYAN_PAIR 	56

#define BLACK_WHITE_PAIR 	57
#define RED_WHITE_PAIR 		58
#define GREEN_WHITE_PAIR 	59
#define YELLOW_WHITE_PAIR 	60
#define BLUE_WHITE_PAIR 	61
#define MAGENTA_WHITE_PAIR 	62
#define CYAN_WHITE_PAIR 	63
#define WHITE_WHITE_PAIR 	64
*/
// end of color pairs

class GraphicsManager : public Manager {
	private:
		// singleton implementation
		GraphicsManager();
		GraphicsManager(const GraphicsManager &gm);
		void operator=(const GraphicsManager &gm);
		
	protected:
		WINDOW *win1, *win2;		// two window buffers for drawing
		WINDOW *curr_win;			// the current buffer
		int window_horiz;			// maximum window width
		int window_vert;			// maximum window height
	
	public:
		~GraphicsManager();
		
		// returns the singleton instance of the GraphicsManager
		static GraphicsManager &getInstance();
		
		// start up the GraphicsManager
		// returns 0 if ok, and -1 on error
		int startup();
		
		// shuts down the GraphicsManager
		void shutdown();
		
		// switch to the other window buffer to display the
		//		next frame
		// return 0 if ok, and -1 on error
		int swapBuffers();
		
		// draw a character at Position world_pos
		// NOTE: (0,0) is at top-left of screen with +y going down
		// Optional color arguments
		// return 0 if ok, and -1 on error
		int drawCh(Position ch_pos, char ch);
		int drawCh(Position ch_pos, char ch, int color);
		
		// arguments: 	world_pos 	- the position to draw the Frame on the screen
		//			 	frame 		- the frame to draw
		//				centered 	- is the frame centered on the position or
		//								is the position in the upper-left corner of the frame
		//				color		- what color pair should be used
		// draw a frame at Position world_pos
		// return 0 if success, and -1 on error
		int drawFrame(Position world_pos, Frame frame, bool centered, int color);
		
		// return window_horiz or window_vert
		int getHorizontal();
		int getVertical();
};// GraphicsManager

#endif //_GRAPHICS_MANAGER_H_
