/*
	LogManager.h
	Written by Cordell Zebrose
	
	LogManager deals with writting to the log and keeping a timestamp for all log entries.
*/

#ifndef _LOG_MANAGER_H_
#define _LOG_MANAGER_H_

#include "Manager.h"
#include <stdio.h>

#define LOGFILE_NAME "fumes.log"

#define DEBUG1 	1	// Basic Engine Starting and Shutting down Log Messages
#define DEBUG2 	1	// Slightly Detailed Engine Log Messages
//#define DEBUG3 	1 	// Highly Detailed Engine Log Messages && User-level Log Messages

class LogManager : public Manager {
	private:
		LogManager(const LogManager &lm);		// prevent copying
		void operator=(const LogManager &lm);	// prevent copying
		LogManager();							// prevent more than one LogManager()
	
	protected:
		bool do_flush;			// true if the output buffer is flushed after every log entry
		FILE *p_file;			// handle for the log file output
		char *getTimeString();	// modifies the argument retval to be the time in a nice format
	
	public:
		~LogManager();
		
		// returns the LogManager
		// 	required for singleton implementation
		static LogManager &getInstance();
		
		// Starts the Log Manager
		// 	Opens the log file
		int startup();
		int startup(bool append, bool flush);
		
		// Shuts down the Log Manager
		//	Closes the log file
		void shutdown();
		
		// writes a message to the log with an associated timestamp
		//	the arguments are formatted in the same way as a printf()
		//	returns the number of bytes written, or -1 on error
		int writeLog(const char *msg, ...);
	
};

#endif // _LOG_MANAGER_H_
