/*
	Manager.h
	Written by Cordell Zebrose
	
	Manager is the base class for all the other Managers. It contains
		default implementations of required members of any Manager class such
		as startup() and shutdown().
*/

#ifndef _MANAGER_H_
#define _MANAGER_H_

#include "GameObject.h"
#include "GameObjectList.h"
#include "GameObjectListIterator.h"
#include <string.h>

#define MAX_EVENTS 1000

class Manager {
	protected:
		bool is_started;
		
		// event variables
		string event[MAX_EVENTS];  // store all the known event types
		GameObjectList event_obj_list[MAX_EVENTS]; // store interested objects
		int num_events;
		
	public:
		Manager();
		virtual ~Manager();
		
		// Start the Manager
		// returns 0 if started up, otherwise -1
		virtual int startup();
		
		// Shutdown the Manager
		virtual void shutdown();
		
		// returns true if the Manager has started and not shutdown
		bool isStarted();
		
		// returns true if event is allowed by Manager, otherwise false
		// base class always returns true
		virtual bool isValid(string event_name);
		
		// send event p_e to all interested objects
		void onEvent(Event *p_e);
		
		// register GameObject p_go with Event event_name
		// return 0 on ok, and -1 on error
		int registerInterest(GameObject *p_go, string event_name);
		
		// unregister GameObject p_go with Event event_name
		// Return 0 on ok, and -1 on error
		int unregisterInterest(GameObject *p_go, string event_name);
};

#endif // _MANAGER_H_
