/*
	ResourceManager.h
	Written by Cordell Zebrose
	
	ResourceManager handles all 
*/

#include "Manager.h"
#include "Sprite.h"
#include "Frame.h"

#include <string.h>

#define MAX_SPRITES 100
#define MAX_FRAMES_PER_SPRITE 60

class ResourceManager : public Manager {
	private:
		// singleton implementation
		ResourceManager();
		ResourceManager(const ResourceManager &rm);
		void operator=(const ResourceManager &rm);
	
	protected:
		Sprite *sprite[MAX_SPRITES];
		int sprite_count;
	
	public:
		~ResourceManager();
		
		// return the only instance of ResourceManager
		static ResourceManager &getInstance();
		
		// start up the ResourceManager
		// returns 0 if ok, and -1 on error
		int startup();
		
		// shuts down the ResourceManager
		void shutdown();
		
		// argument: 	filename - the file to open
		//				label 	 - the label for the sprite
		// Loads the Sprite from the file and gives it
		//	a label.
		// returns 0 if successful, or -1 on error
		int loadSprite(string filename, string label);
		
		// argument: label - the label on the requested Sprite
		// returns the Sprite with the label given
		Sprite *getSprite(string label);
		
		// show the sprite_count
		int getSpriteCount();
}; // InputManager
