/*
  GameObjectList.cpp
  Written by Cordell Zebrose
*/

#include "GameObjectList.h"
#include "LogManager.h"

/////////////////////////////////////////////////////////////////////

// arguements: none
// Creates a GameObjectListIterator for this list and returns it?
GameObjectListIterator GameObjectList::createIterator() const {
	GameObjectListIterator iter(this);
	
	return iter;
} // createIterator()
		
//////////////////////////////////////////////////////////////////////

// default constructor for a GameObjectList, creates an empty list
GameObjectList::GameObjectList() {
	clear();
}// GameObjectList()

/////////////////////////////////////////////////////////////////////////

GameObjectList::~GameObjectList() {
}// ~GameObjectList()

////////////////////////////////////////////////////////////////////////
		
// arguments: p_go - pointer to the GameObject to be added
// Inserts a game object into the list
// Return 0 if ok, or -1 on error
int GameObjectList::insert(GameObject *p_go) {
	// check size of the array
	if( count >= MAX_GAME_OBJECTS) 
	{
		return -1;
	}// if
	
	for(int i = 0; i < count; i++)
	{
		if(list[i] == p_go) {
			return 0;
		}// if
	}// for
	
	list[count] = p_go;
	count++;
	return 0;
}// insert

/////////////////////////////////////////////////////////////////////////

// arguments: p_go - pointer to the GameObject to be removed
// Remove a game object from the list which shares an id with p_go
// Return 0 if removed, or -1 if not found
int GameObjectList::remove(GameObject *p_go) {
	LogManager &lm = LogManager::getInstance();
	
	for(int i=0; i < count; i++)
	{
		// check that the list is intact
		if(list[i] == NULL)
		{
#ifdef DEBUG1
			lm.writeLog("GameObjectList::remove: Error - There is a NULL pointer in the middle of the list at location %d, \
							while there are reported %d items in the list.", i, count);
#endif
		}else if( p_go->getId() == list[i]->getId())
		{
			// found the item
			for(int j=i; j < count; j++)
			{
				list[j] = list[j+1];
			}// for
			count--;
			return 0;
		}// if-elseif
	}//for
	
	// didn't find the item
	return -1;
}// remove
		
////////////////////////////////////////////////////////////////////////////

// Clear the list (setting count to 0)
void GameObjectList::clear() {
	count = 0;
	
	for(int i = 0; i < MAX_GAME_OBJECTS; i++)
		list[i] = NULL;
}// clear()

/////////////////////////////////////////////////////////////////////////
		
// argument: id - the id of the GameObject to find
// Return pointer to object with indicated id,
//   or NULL if not found
GameObject *GameObjectList::find(int id) {
	for(int i=0; i < count; i++)
	{
		if(list[i] == NULL)
		{
			LogManager &lm = LogManager::getInstance();
#ifdef ERROR
			lm.writeLog("GameObjectList::remove: Error - There is a NULL pointer in the middle of the list at location %d, \
							while there are reported %d items in the list.", i, count);
#endif
		}else if(list[i]->getId() == id)
		{
			return list[i];
		}// if-elseif
	}// for
	
	// GameObject not found
	return NULL;
}// find
		
///////////////////////////////////////////////////////////////////////////

// Return count of number of objects in list
int GameObjectList::getCount() {
	return count;
}// getCount
