/*
  GameObjectListIterator.cpp
  Written by Cordell Zebrose
*/

#include "GameObjectListIterator.h"
#include "LogManager.h"

///////////////////////////////////////////////////////

// The default constructor should never be called
GameObjectListIterator::GameObjectListIterator() {}

///////////////////////////////////////////////////////

// Nothing to free from memory
GameObjectListIterator::~GameObjectListIterator() {
}

////////////////////////////////////////////////////////

GameObjectListIterator::GameObjectListIterator(const GameObjectList *p_L) {
	index = 0;
	p_list = p_L;
}// GameObjectListIterator()

///////////////////////////////////////////////////////////

void GameObjectListIterator::first() {
	index = 0;
}// first

///////////////////////////////////////////////////////////

void GameObjectListIterator::next() {
	index++;
}//next()

//////////////////////////////////////////////////////////////

bool GameObjectListIterator::isDone() {
	if(index < p_list->count) 
	{
		return false;
	}else if(index == p_list->count)
	{
		return true;
	}else
	{
		LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
		lm.writeLog("GameObjectListIterator::isDone(): index went off the end of the array to location %d", index);
#endif
		index = p_list->count;
		return true;
	}// if-elseif-else
}// isDone

//////////////////////////////////////////////////////////////////

GameObject *GameObjectListIterator::currentObj() {
	LogManager &lm = LogManager::getInstance();
	
	// make sure the index hasn't fallen off the end of the array
	if(isDone())
	{
#ifdef DEBUG2
		lm.writeLog("GameObjectListIterator::currentObj: The iterator went off the end of the list, make \
						sure to check isDone() before calling currentObj()");
		return NULL;
#endif
	}//if
	
	if(p_list == NULL)
	{
#ifdef DEBUG2
		lm.writeLog("GameObjectListIterator::currentObj: This iterator has no list");
#endif
		return NULL;
	}else if(p_list->list == NULL)
	{
#ifdef DEBUG2
		lm.writeLog("GameObjectListIterator::currentObj: This iterator's list has an empty list");
#endif
		return NULL;
	}else if(p_list->list[index] == NULL)
	{
#ifdef DEBUG2
		lm.writeLog("GameObjectListIterator::currentObj: index is currently at a NULL pointer in the list");
#endif
		return NULL;
	}else
	{
		return p_list->list[index];
	}// if-elseif-elseif-else
}// currentObj
