/*
	LogManager.cpp
	Written by Cordell Zebrose
*/

// included Engine Files
#include "LogManager.h"

// standard include files
#include <string.h>
#include <time.h>
#include <stdarg.h>

/////////////////////////////////////////////////////////////////////

// made private to prevent the user from copying the LogManager instance
LogManager::LogManager(const LogManager &lm) {
} // LogManager()

//////////////////////////////////////////////////////////////////////

// made private to prevent the user from copying the LogManager instance
void LogManager::operator=(const LogManager &lm) {
} // operator=()

///////////////////////////////////////////////////////////////////////

LogManager::LogManager() {
} // LogManager()

//////////////////////////////////////////////////////////////////////////
	
LogManager::~LogManager() {
	if(is_started)
		shutdown();
} // LogManager()

//////////////////////////////////////////////////////////////////////
	
// arguments: retval - this a pointer to some space in memory which will
//						be changed by this function
// sets the retval pointer to the time into a well-formatted string
char *LogManager::getTimeString() {
	static char time_str[30];
	struct tm *p_time;
	time_t cTime;
	
	time(&cTime);
	p_time = localtime(&cTime);
	
	int error = 
	sprintf(time_str, "%02d:%02d:%02d",
		p_time->tm_hour,
		p_time->tm_min,
		p_time->tm_sec);
	
	if(error < 0)
	{
		sprintf(time_str, "99:99:99");
		return time_str;
	}
	
	return time_str;
} // getTimeString

/////////////////////////////////////////////////////////////////////////
		
// returns the LogManager
// 	required for singleton implementation
LogManager &LogManager::getInstance() {
	static LogManager log_manager;
	return log_manager;
}

//////////////////////////////////////////////////////////////////////////
		
// Start the Log Manager with default arguments
int LogManager::startup() {
	return startup(true, true);
}
		
//////////////////////////////////////////////////////////////////////////

// Starts the Log Manager
// 	Opens the log file with the append tag, unless the append argument is false
//	if the flush argument is true, then the output buffer is flushed after every
//		log write.
int LogManager::startup(bool append, bool flush) {
	if(is_started)
	{
		return 0;
	}// if
	
	if(append)
	{
		p_file = fopen(LOGFILE_NAME, "a");
	}else
	{
		p_file = fopen(LOGFILE_NAME, "w");
	}// if-else
		
	fprintf(p_file, "\n\n==================================================================\n");
	fprintf(p_file, "%s:LogManager::startUp: Log file has opened\n", getTimeString());
	do_flush = flush;
	
	is_started = true;
	return 0;
}// LogManager::startup

/////////////////////////////////////////////////////////////////////////
		
// Shuts down the Log Manager
//	Closes the log file
void LogManager::shutdown() {
	if(p_file != NULL)
	{
		fprintf(p_file, "%s:LogManager::shutdown: Log file has closed\n", getTimeString());
		fprintf(p_file, "==================================================================\n\n");
		fclose(p_file);
	}
	
	is_started = false;
}

/////////////////////////////////////////////////////////////////////////
		
// writes a message to the log with an associated timestamp
//	the arguments are formatted in the same way as a printf()
//	returns the number of bytes written, or -1 on error
int LogManager::writeLog(const char *msg, ...) {
	fprintf(p_file, "%s:", getTimeString()); // print the time stamp
	
	// print the message
	va_list args;
	int total_bytes = 0;
	va_start( args, msg);
	int curr_bytes = vfprintf(p_file, msg, args );
	if(curr_bytes < 0)
	{
		fprintf(p_file, "LogManager::writeLog: Error: message couldn't finish printing\n");
		
		return -1;
	}// if
	total_bytes += curr_bytes;
	va_end( args );
	
	// add newline and return number of bytes printed
	fprintf(p_file, "\n");
	if(do_flush)
	{
		fflush(p_file);
	}// if
	return total_bytes;
}

//////////////////////////////////////////////////////////////////////////
