/*
	Manager.cpp
	Written by Cordell Zebrose
*/

#include "Manager.h"
#include "LogManager.h"

//////////////////////////////////////////////////////////////////////

Manager::Manager() {
}// Manager::Manager

//////////////////////////////////////////////////////////////////////

Manager::~Manager() {
}// Manager::~Manager

//////////////////////////////////////////////////////////////////////
		
// Start the Manager
// returns 0 if started up, otherwise -1
int Manager::startup() {
	is_started = true;
	num_events = 0;
	return 0;
}// Manager::startup

//////////////////////////////////////////////////////////////////////
		
// Shutdown the Manager
void Manager::shutdown() {
	is_started = false;
}// Manager::shutdown

//////////////////////////////////////////////////////////////////////

bool Manager::isStarted() {
	return is_started;
}// Manager::isStated

///////////////////////////////////////////////////////////////////////

// arguments: p_e - A pointer to an instance of the Event which was generated
// send event p_e to all interested objects
void Manager::onEvent(Event *p_e) {
	// find the event in event[]
	int event_index;
	for(event_index = 0; event_index < num_events; event_index++)
	{
		if(event[event_index] == p_e->getType())
			break;
	}// for
	
	// check to see if the event was recognized
	if(event_index >= num_events)
	{
		LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
		lm.writeLog("Manager::onEvent: Unrecognized %s Event", p_e->getType().c_str());
#endif
		return;		
	}// if
	
	// create iterator
	GameObjectListIterator iter = event_obj_list[event_index].createIterator();
	
	while(!iter.isDone())
	{	
		LogManager &lm = LogManager::getInstance();
	
		// grab current GameObject
		GameObject *curr = iter.currentObj();
		
		if(curr)
		{
#ifdef DEBUG3
			lm.writeLog("Manager::onEvent: Sending %s Event to %s #%d", p_e->getType().c_str(), curr->getType().c_str() ,curr->getId());
#endif
				
			// send the current GameObject an event
			curr->eventHandler(p_e);
		}else
		{
#ifdef DEBUG1
			lm.writeLog("Manager::onEvent: Error - the iterator returned a NULL pointer");
#endif
		}//if-else
		
		// iterate to the next GameObject
		iter.next();
	}// while
}//onEvent
		
///////////////////////////////////////////////////////////////////////////////////		
		
// returns true if event is allowed by Manager, otherwise false
// base class always returns true
bool Manager::isValid(string event_name) {	
	return true;
}// isValid
		
///////////////////////////////////////////////////////////////////////////////////////
		
// register GameObject p_go with Event event_name
// return 0 on ok, and -1 on error
int Manager::registerInterest(GameObject *p_go, string event_name) {	
	LogManager &lm = LogManager::getInstance();
	
	if(!isValid(event_name))
	{
#ifdef DEBUG3
		lm.writeLog("Manager::registerInterest: This Manager does not allow '%s' events to be registered with it", 
					event_name.c_str());
#endif
		return -1;
	}// if
	
	// check if the event is already add
	for(int i = 0; i < num_events; i++) {
		if(event[i] == event_name)
		{
#ifdef DEBUG3
			lm.writeLog("Manager::registerInterest: %s #%d is registering interest in event %s",
						p_go->getType().c_str(), p_go->getId(), event_name.c_str());
#endif
			return event_obj_list[i].insert(p_go);
		}
	}// for

	// the event is new
	if(num_events < MAX_EVENTS) {
#ifdef DEBUG2
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("Manager::registerInterest(): new %s event created", event_name.c_str());
#endif
		event[num_events] = event_name;
		
#ifdef DEBUG3
		lm.writeLog("Manager::registerInterest: %s #%d is registering interest in event %s",
					p_go->getType().c_str(), p_go->getId(), event_name.c_str());
#endif
		
		return event_obj_list[num_events++].insert(p_go);
	}else
	{
		// maximum number of events reached
#ifdef DEBUG2
		LogManager &lm = LogManager::getInstance();
		lm.writeLog("Manager::registerInterest(): Maximum number of events reached - \
						new Event %s is not added", event_name.c_str());
#endif
		return -1;		
	}// if-else
}// registerInterest
		
//////////////////////////////////////////////////////////////////////////////////////
		
// unregister GameObject p_go with Event event_name
// Return 0 on ok, and -1 on error
int Manager::unregisterInterest(GameObject *p_go, string event_name) {
	// check for event
	for(int i = 0; i < num_events; i++)
	{
		if(event[i] == event_name)
		{
#ifdef DEBUG3
			LogManager &lm = LogManager::getInstance();
			lm.writeLog("Manager::unregisterInterest: %s #%d is unregistering interest in event %s",
						p_go->getType().c_str(), p_go->getId(), event_name.c_str());
#endif
		
			return event_obj_list[i].remove(p_go);
		}//if
	}// for
	
	// event doesn't exist
	LogManager &lm = LogManager::getInstance();
#ifdef DEBUG2
	lm.writeLog("Manager::unregisterInterest: Unknown event %s", event_name.c_str());
#endif
	return -1;
}// unregisterInterest()
