/*
  Sprite.cpp
  Written by Cordell Zebrose
*/

#include "Sprite.h"
#include "GraphicsManager.h"

/////////////////////////////////////////////////////////

// protected constructor
//	shouldn't be called
Sprite::Sprite() {
}// Sprite
		
//////////////////////////////////////////////////////////

// max_frames is just the maximum number of
//		frames a Sprite can have.
Sprite::Sprite(int max_frames) {
	width = 0;
	height = 0;
	max_frame_count = max_frames;
	frame_count = 0;
	frame_index = 0;
	color = WHITE_BLACK_PAIR;
	frame = new Frame[max_frames];
	label = "Sprite";
}// Sprite
		
/////////////////////////////////////////////////////////////	
	
// same as getFrame(frame_index)
Frame Sprite::getFrame() {
	return getFrame(frame_index);
}// getFrame
	
// Returns the frame #'frame_number'
//   i.e. frame_number = 0 will return the first frame, 
//		  frame_number = 1 will return the second frame, etc...
Frame Sprite::getFrame(int frame_number) {
	if(frame_number < 0)
	{
		Frame empty_frame;
		return empty_frame;
	}else if(frame_number < frame_count)
	{
		return frame[frame_number];
	}else
	{
		Frame empty_frame;
		return empty_frame;
	}// if-else if-else
}// getFrame
		
/////////////////////////////////////////////////////////////////

// Add the new Frame to the end of the Sprite
// returns 0 if successful, or -1 if failed
int Sprite::addFrame(Frame new_frame) {
	if(frame_count < max_frame_count)
	{
		frame[frame_count] = new_frame;
		frame_count++;
		return 0;
	}else
	{
		return -1;
	}//if-else
}// addFrame
	
/////////////////////////////////////////////////////////////////////
	
int Sprite::getWidth() {
	return width;
}// getWidth

int Sprite::getHeight() {
	return height;
}// getHeight

std::string Sprite::getLabel() {
	return label;
}// getLabel

int Sprite::getColor() {
	return color;
}// getColor

int Sprite::getFrameCount() {
	return frame_count;
}// getFrameCount

int Sprite::getFrameIndex() {
	return frame_index;
}// getFrameIndex
		
void Sprite::setWidth(int new_width) {
	width = new_width;
}// setWidth

void Sprite::setHeight(int new_height) {
	height = new_height;
}// setHeight

void Sprite::setLabel(std::string new_label) {
	label = new_label;
}// setLabel

void Sprite::setColor(int new_color) {
	color = new_color;
}// setColor

void Sprite::setFrameIndex(int new_frame_index) {
	if(new_frame_index < 0)
	{
		frame_index = 0;
	}else if(new_frame_index < frame_count)
	{
		frame_index = new_frame_index;
	}else
	{
		frame_index = 0;
	}
}// setFrameIndex
